//SAADC
#define SAADC_SAMPLES_IN_BUFFER 								1
#define SAADC_OVERSAMPLE										NRF_SAADC_OVERSAMPLE_DISABLED
#define SAADC_BURST_MODE										0

static bool m_saadc_initialized = false; 
static bool m_saadc_uninit = false;
static nrf_saadc_value_t m_buffer_pool[2];

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*
	APP_TIMER IDENTIFIERS 
*/
APP_TIMER_DEF(saadc_timer_id);

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

static void lfclk_config(void) {
	
	uint32_t err_code = nrf_drv_clock_init();
	APP_ERROR_CHECK(err_code);
	nrf_drv_clock_lfclk_request(NULL);
}

void saadc_event_handler(nrf_drv_saadc_evt_t const * p_event)
{
	if (p_event->type == NRF_DRV_SAADC_EVT_DONE)
	{
		SEGGER_RTT_WriteString(0, "SAADC sample obtained!\n\n");
		
		ret_code_t err_code;
		/*
			SET THE BUFFER 1 OR 2 SO THE SAADC CAN BE WRITEN TO AGAIN
		*/
		err_code = nrf_drv_saadc_buffer_convert(p_event->data.done.p_buffer, SAADC_SAMPLES_IN_BUFFER);
		APP_ERROR_CHECK(err_code);
		
		uint32_t _int_buffer_data;
		char _char_buffer_data[20];
		//for (int i = 0; i < p_event->data.done.size; i++)
		{
			_int_buffer_data = p_event->data.done.p_buffer[0];
			sprintf(_char_buffer_data, "%d\n\n", _int_buffer_data);
			SEGGER_RTT_WriteString(0, _char_buffer_data);
		}
		/*
			UNINITIALIZE THE SAADC TO DISABLE EasyDMA AND SAVE POWER
			DISABLE AND CLEAR THE SAADC INTERRUPTS
		*/
		nrf_drv_saadc_uninit();
		NRF_SAADC->INTENCLR = (SAADC_INTENCLR_END_Clear << SAADC_INTENCLR_END_Pos);
		NVIC_ClearPendingIRQ(SAADC_IRQn);
		m_saadc_initialized = false;
		//m_saadc_uninit = true;
	}
}

static void sdaac_init(void)
{
	ret_code_t err_code;
	nrf_drv_saadc_config_t saadc_config;
	nrf_saadc_channel_config_t saadc_channel_config;
	/*
		CONFIGURE THE ADC
	*/
	//saadc_config.low_power_mode = true;
	saadc_config.resolution = NRF_SAADC_RESOLUTION_12BIT;
	saadc_config.oversample = SAADC_OVERSAMPLE;
	saadc_config.interrupt_priority = APP_IRQ_PRIORITY_LOW;
	/*
		INITIALIZE THE SAADC
	*/
	err_code = nrf_drv_saadc_init(&saadc_config, saadc_event_handler);
	APP_ERROR_CHECK(err_code);
	/*
		CONFIGURE THE SAADC CHANNEL
	*/
	saadc_channel_config.resistor_p = NRF_SAADC_RESISTOR_DISABLED;
	saadc_channel_config.resistor_n = NRF_SAADC_RESISTOR_DISABLED;
	saadc_channel_config.gain = NRF_SAADC_GAIN1_4;
	saadc_channel_config.reference = NRF_SAADC_REFERENCE_INTERNAL;
	saadc_channel_config.acq_time = NRF_SAADC_ACQTIME_10US;
	saadc_channel_config.mode = NRF_SAADC_MODE_SINGLE_ENDED;
	saadc_channel_config.pin_p = NRF_SAADC_INPUT_AIN0;
	saadc_channel_config.pin_n = NRF_SAADC_INPUT_DISABLED;
	/*
		INITIALIZE THE SAADC CHANNEL
	*/
	err_code = nrf_drv_saadc_channel_init(0, &saadc_channel_config);
	APP_ERROR_CHECK(err_code);
	/*
		ACTIVATE BURST MODE
	*/
	if(SAADC_BURST_MODE)
	{
			NRF_SAADC->CH[0].CONFIG |= 0x01000000;
	}
	/*
		SET BUFFER 1 TO WHERE THE SAADC WILL WRITE TO
	*/
	err_code = nrf_drv_saadc_buffer_convert(&m_buffer_pool[0], SAADC_SAMPLES_IN_BUFFER);
	APP_ERROR_CHECK(err_code);
	/*
		SET BUFFER 2 TO WHERE THE SAADC WILL WRITE TO WHEN BUFFER 1 IS FULL
	*/
	err_code = nrf_drv_saadc_buffer_convert(&m_buffer_pool[1], SAADC_SAMPLES_IN_BUFFER);
	APP_ERROR_CHECK(err_code);
}

static void saadc_timer_handler(void * p_context)
{
	SEGGER_RTT_WriteString(0, "SAADC sample requested!\n\n");
	/*
		IF THE SAADC WAS UNINITIALIZED THEN IT SHOULD BE INITIALIZED AGAIN BEFORE NEW SAMPLE REQUEST
	*/
	if(!m_saadc_initialized)
	{
			sdaac_init();
	}
	m_saadc_initialized = true; 
	
	nrf_drv_saadc_sample();
}
/** @} */

/**
 * @brief Function for application main entry.
 */
int main(void)
{
	lfclk_config();
	rtc_config();
	
	input_init();
	output_init();
	//sdaac_init();
	
	SEGGER_RTT_WriteString(0, "nRF_remote program started!\n\n");

	uint32_t err_code;
	/*
		INITIATE SCHEDULER
	*/
	APP_SCHED_INIT(SCHEDULER_MAX_EVENT_DATA_SIZE, SCHEDULER_QUEUE_SIZE);
	/*
		INITIATE THE APP_TIMER MODULE
		NOT USED AS WE ARE GOING TO USE THE APP_TIMER COMBINED WITH THE SCHEDULER
	*/
	//APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_OP_QUEUE_SIZE, false);
	/*
		INITIATE THE APP_TIMER TO BE USED WITH THE SCHEDULER
	*/
	APP_TIMER_APPSH_INIT(APP_TIMER_PRESCALER, APP_TIMER_OP_QUEUE_SIZE, false);
	/*
		INITIATE THE APP_BUTTON CONFIGURING IT TO DETECT SHORT PRESSES OF THE KEYPAD BUTTONS
	*/
	err_code = app_button_init((app_button_cfg_t *)app_buttons, sizeof(app_buttons) / sizeof(app_buttons[0]), APP_TIMER_TICKS(APP_BUTTON_SHORT_PRESS_DEBOUNCE, APP_TIMER_PRESCALER));
	APP_ERROR_CHECK(err_code);
	/*
		ENABLE BUTTON DETECTION
	*/
	err_code = app_button_enable();
	APP_ERROR_CHECK(err_code);
	/*
		CREATE A REPEATED TIMER THAT WHEN EXPIRES TRIGGERS A SAADC SAMPLE
	*/
	err_code = app_timer_create(&saadc_timer_id, APP_TIMER_MODE_REPEATED, saadc_timer_handler);
	APP_ERROR_CHECK(err_code);
	/*
		START THE TIMER AND SET THE TIMEOUT TO BE EVERY "BATT_READING_DELAY"
	*/
	err_code = app_timer_start(saadc_timer_id, APP_TIMER_TICKS(BATT_READING_DELAY, APP_TIMER_PRESCALER), NULL);
	APP_ERROR_CHECK(err_code);
	
	while (true)
	{
		app_sched_execute();
		
		if (m_saadc_uninit)
		{
			nrf_drv_saadc_uninit();
			NRF_SAADC->INTENCLR = (SAADC_INTENCLR_END_Clear << SAADC_INTENCLR_END_Pos);
			NVIC_ClearPendingIRQ(SAADC_IRQn);
			m_saadc_uninit = false;
			m_saadc_initialized = false;
		}
	}
}
/** @} */
